#include <avr/io.h>
#include "app4.h"
#include "app4delay.h"
#include <avr/pgmspace.h>
#include <avr/signal.h>
#include <avr/interrupt.h>

#ifndef RELEASE  // I'm only using RS232, stdio for debugging
#include <stdio.h>
#include "app4uart.h"
#define DEBUG(s) s
#else
#define DEBUG(s)
#endif

#include <string.h>
#include <ctype.h>
#include "app4io.h"


/* dit delay for morse code */
#define DITDELAY 50  /* mS */

/* Output on PORTD.7 */
#define OUTPORT D
#define OUTBIT 7
/* Speaker on PORTB.7 */
#define SPKRPORT B
#define SPKRBIT 0 

void timer_init(void)
{
  TCCR0=2;  // 10MHz clock direct prescaled by 8
  TIMSK|=1; // enable 
  sei();    // enable all interrupts
}

// this will be about 4.8kHz so divide by 4
INTERRUPT(SIG_OVERFLOW0)
{
  static unsigned divider=0;
  if (divider++%2) return; // divide by 2 to get about 2400Hz (each half cycle)
  PORTB^=1; // toggle pin (only matters if output is set)
}


void tone(int dlay)
{
  DDRB|=1;
  delay_ms(dlay*DITDELAY);
  DDRB&=~1;
}


/* Send a morse code element */
void send(char c)
{
  switch (c)
    {
    case '-':
      HIGH(OUTPORT,OUTBIT);
      tone(3);
      LOW(OUTPORT,OUTBIT);
      break;
    case '.':
      HIGH(OUTPORT,OUTBIT);
      tone(1);
      LOW(OUTPORT,OUTBIT);
      break;
    case ' ':
      delay_ms(4*DITDELAY);
      break;
    }
  delay_ms(DITDELAY);  /* inter element space */
}

void sendc(PGM_P s)
{
  int c;
  for (c=pgm_read_byte(s);c;c=pgm_read_byte(++s)) 
    {
      DEBUG(printf("element@%d: %c\n",s,c));
      send(c);
    }
}

void sendchar(int c)
{
  DEBUG(printf("Send %c\n",c));
  switch (toupper(c))
    {
    case ' ': sendc(PSTR(" ")); break;
    case 'A': sendc(PSTR(".-")); break;
    case 'C': sendc(PSTR("-.-.")); break;
    case 'D': sendc(PSTR("-..")); break;
    case 'E': sendc(PSTR(".")); break;
    case 'G': sendc(PSTR("--.")); break;
    case 'K': sendc(PSTR("-.-")); break;
    case 'N': sendc(PSTR("-.")); break;
    case 'Q': sendc(PSTR("--.-")); break;
    case 'R': sendc(PSTR(".-.")); break;
    case 'W': sendc(PSTR(".--")); break;
    case '5': sendc(PSTR(".....")); break;
    }
  send(' ');
}



/* Main program */
int main (void)
{
PGM_P message=PSTR("CQ CQ CQ de WD5GNR WD5GNR WD5GNR K");

#ifndef RELEASE
   UartInit(BAUD_19200);
   UartSetStdio();
   printf_P(PSTR("\014Debug: CW Beacon starting\n"));
#endif


 LOW(OUTPORT,OUTBIT);   // Make LED an output
 // We use the timer to generate a 600Hz tone
 // and then we turn the output bit on to gate the tone on
 // and off to silence it
 timer_init(); 

 while (1) 
   {
     int c;
     PGM_P ptr=message;
     for (c=pgm_read_byte(ptr);c;c=pgm_read_byte(++ptr))
       {
         sendchar(c);
       }

     delay_ms(30000);
   }
  return 0;
}

